'use strict';

var alias$1 = require('../alias-aa343af5.cjs');
var session = require('../session-87c6f58b.cjs');

class SQLiteBigIntBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteBigIntBuilder';
    constructor(name) {
        super(name, 'bigint', 'SQLiteBigInt');
    }
    /** @internal */
    build(table) {
        return new SQLiteBigInt(table, this.config);
    }
}
class SQLiteBigInt extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteBigInt';
    getSQLType() {
        return 'blob';
    }
    mapFromDriverValue(value) {
        return BigInt(value.toString());
    }
    mapToDriverValue(value) {
        return Buffer.from(value.toString());
    }
}
class SQLiteBlobJsonBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteBlobJsonBuilder';
    constructor(name) {
        super(name, 'json', 'SQLiteBlobJson');
    }
    /** @internal */
    build(table) {
        return new SQLiteBlobJson(table, this.config);
    }
}
class SQLiteBlobJson extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteBlobJson';
    getSQLType() {
        return 'blob';
    }
    mapFromDriverValue(value) {
        return JSON.parse(value.toString());
    }
    mapToDriverValue(value) {
        return Buffer.from(JSON.stringify(value));
    }
}
class SQLiteBlobBufferBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteBlobBufferBuilder';
    constructor(name) {
        super(name, 'buffer', 'SQLiteBlobBuffer');
    }
    /** @internal */
    build(table) {
        return new SQLiteBlobBuffer(table, this.config);
    }
}
class SQLiteBlobBuffer extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteBlobBuffer';
    getSQLType() {
        return 'blob';
    }
}
function blob(name, config) {
    if (config?.mode === 'json') {
        return new SQLiteBlobJsonBuilder(name);
    }
    if (config?.mode === 'bigint') {
        return new SQLiteBigIntBuilder(name);
    }
    return new SQLiteBlobBufferBuilder(name);
}

class SQLiteCustomColumnBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteCustomColumnBuilder';
    constructor(name, fieldConfig, customTypeParams) {
        super(name, 'custom', 'SQLiteCustomColumn');
        this.config.fieldConfig = fieldConfig;
        this.config.customTypeParams = customTypeParams;
    }
    /** @internal */
    build(table) {
        return new SQLiteCustomColumn(table, this.config);
    }
}
class SQLiteCustomColumn extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteCustomColumn';
    sqlName;
    mapTo;
    mapFrom;
    constructor(table, config) {
        super(table, config);
        this.sqlName = config.customTypeParams.dataType(config.fieldConfig);
        this.mapTo = config.customTypeParams.toDriver;
        this.mapFrom = config.customTypeParams.fromDriver;
    }
    getSQLType() {
        return this.sqlName;
    }
    mapFromDriverValue(value) {
        return typeof this.mapFrom === 'function' ? this.mapFrom(value) : value;
    }
    mapToDriverValue(value) {
        return typeof this.mapTo === 'function' ? this.mapTo(value) : value;
    }
}
/**
 * Custom sqlite database data type generator
 */
function customType(customTypeParams) {
    return (dbName, fieldConfig) => {
        return new SQLiteCustomColumnBuilder(dbName, fieldConfig, customTypeParams);
    };
}

class SQLiteBaseIntegerBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteBaseIntegerBuilder';
    constructor(name, dataType, columnType) {
        super(name, dataType, columnType);
        this.config.autoIncrement = false;
    }
    primaryKey(config) {
        if (config?.autoIncrement) {
            this.config.autoIncrement = true;
        }
        this.config.hasDefault = true;
        return super.primaryKey();
    }
}
class SQLiteBaseInteger extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteBaseInteger';
    autoIncrement = this.config.autoIncrement;
    getSQLType() {
        return 'integer';
    }
}
class SQLiteIntegerBuilder extends SQLiteBaseIntegerBuilder {
    static [alias$1.entityKind] = 'SQLiteIntegerBuilder';
    constructor(name) {
        super(name, 'number', 'SQLiteInteger');
    }
    build(table) {
        return new SQLiteInteger(table, this.config);
    }
}
class SQLiteInteger extends SQLiteBaseInteger {
    static [alias$1.entityKind] = 'SQLiteInteger';
}
class SQLiteTimestampBuilder extends SQLiteBaseIntegerBuilder {
    static [alias$1.entityKind] = 'SQLiteTimestampBuilder';
    constructor(name, mode) {
        super(name, 'date', 'SQLiteTimestamp');
        this.config.mode = mode;
    }
    /**
     * @deprecated Use `default()` with your own expression instead.
     *
     * Adds `DEFAULT (cast((julianday('now') - 2440587.5)*86400000 as integer))` to the column, which is the current epoch timestamp in milliseconds.
     */
    defaultNow() {
        return this.default(alias$1.sql `(cast((julianday('now') - 2440587.5)*86400000 as integer))`);
    }
    build(table) {
        return new SQLiteTimestamp(table, this.config);
    }
}
class SQLiteTimestamp extends SQLiteBaseInteger {
    static [alias$1.entityKind] = 'SQLiteTimestamp';
    mode = this.config.mode;
    mapFromDriverValue(value) {
        if (this.config.mode === 'timestamp') {
            return new Date(value * 1000);
        }
        return new Date(value);
    }
    mapToDriverValue(value) {
        const unix = value.getTime();
        if (this.config.mode === 'timestamp') {
            return Math.floor(unix / 1000);
        }
        return unix;
    }
}
class SQLiteBooleanBuilder extends SQLiteBaseIntegerBuilder {
    static [alias$1.entityKind] = 'SQLiteBooleanBuilder';
    constructor(name, mode) {
        super(name, 'boolean', 'SQLiteBoolean');
        this.config.mode = mode;
    }
    build(table) {
        return new SQLiteBoolean(table, this.config);
    }
}
class SQLiteBoolean extends SQLiteBaseInteger {
    static [alias$1.entityKind] = 'SQLiteBoolean';
    mode = this.config.mode;
    mapFromDriverValue(value) {
        return Number(value) === 1;
    }
    mapToDriverValue(value) {
        return value ? 1 : 0;
    }
}
function integer(name, config) {
    if (config?.mode === 'timestamp' || config?.mode === 'timestamp_ms') {
        return new SQLiteTimestampBuilder(name, config.mode);
    }
    if (config?.mode === 'boolean') {
        return new SQLiteBooleanBuilder(name, config.mode);
    }
    return new SQLiteIntegerBuilder(name);
}
const int = integer;

class SQLiteNumericBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteNumericBuilder';
    constructor(name) {
        super(name, 'string', 'SQLiteNumeric');
    }
    /** @internal */
    build(table) {
        return new SQLiteNumeric(table, this.config);
    }
}
class SQLiteNumeric extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteNumeric';
    getSQLType() {
        return 'numeric';
    }
}
function numeric(name) {
    return new SQLiteNumericBuilder(name);
}

class SQLiteRealBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteRealBuilder';
    constructor(name) {
        super(name, 'number', 'SQLiteReal');
    }
    /** @internal */
    build(table) {
        return new SQLiteReal(table, this.config);
    }
}
class SQLiteReal extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteReal';
    getSQLType() {
        return 'real';
    }
}
function real(name) {
    return new SQLiteRealBuilder(name);
}

class SQLiteTextBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteTextBuilder';
    constructor(name, config) {
        super(name, 'string', 'SQLiteText');
        this.config.enumValues = config.enum;
        this.config.length = config.length;
    }
    /** @internal */
    build(table) {
        return new SQLiteText(table, this.config);
    }
}
class SQLiteText extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteText';
    enumValues = this.config.enumValues;
    length = this.config.length;
    constructor(table, config) {
        super(table, config);
    }
    getSQLType() {
        return `text${this.config.length ? `(${this.config.length})` : ''}`;
    }
}
class SQLiteTextJsonBuilder extends session.SQLiteColumnBuilder {
    static [alias$1.entityKind] = 'SQLiteTextJsonBuilder';
    constructor(name) {
        super(name, 'json', 'SQLiteTextJson');
    }
    /** @internal */
    build(table) {
        return new SQLiteTextJson(table, this.config);
    }
}
class SQLiteTextJson extends session.SQLiteColumn {
    static [alias$1.entityKind] = 'SQLiteTextJson';
    getSQLType() {
        return 'text';
    }
    mapFromDriverValue(value) {
        return JSON.parse(value);
    }
    mapToDriverValue(value) {
        return JSON.stringify(value);
    }
}
function text(name, config = {}) {
    return (config.mode === 'json'
        ? new SQLiteTextJsonBuilder(name)
        : new SQLiteTextBuilder(name, config));
}

function alias(table, alias) {
    return new Proxy(table, new alias$1.TableAliasProxyHandler(alias, false));
}

class CheckBuilder {
    name;
    value;
    static [alias$1.entityKind] = 'SQLiteCheckBuilder';
    brand;
    constructor(name, value) {
        this.name = name;
        this.value = value;
    }
    build(table) {
        return new Check(table, this);
    }
}
class Check {
    table;
    static [alias$1.entityKind] = 'SQLiteCheck';
    name;
    value;
    constructor(table, builder) {
        this.table = table;
        this.name = builder.name;
        this.value = builder.value;
    }
}
function check(name, value) {
    return new CheckBuilder(name, value);
}

class IndexBuilderOn {
    name;
    unique;
    static [alias$1.entityKind] = 'SQLiteIndexBuilderOn';
    constructor(name, unique) {
        this.name = name;
        this.unique = unique;
    }
    on(...columns) {
        return new IndexBuilder(this.name, columns, this.unique);
    }
}
class IndexBuilder {
    static [alias$1.entityKind] = 'SQLiteIndexBuilder';
    /** @internal */
    config;
    constructor(name, columns, unique) {
        this.config = {
            name,
            columns,
            unique,
            where: undefined,
        };
    }
    /**
     * Condition for partial index.
     */
    where(condition) {
        this.config.where = condition;
        return this;
    }
    /** @internal */
    build(table) {
        return new Index(this.config, table);
    }
}
class Index {
    static [alias$1.entityKind] = 'SQLiteIndex';
    config;
    constructor(config, table) {
        this.config = { ...config, table };
    }
}
function index(name) {
    return new IndexBuilderOn(name, false);
}
function uniqueIndex(name) {
    return new IndexBuilderOn(name, true);
}

function primaryKey(...config) {
    if (config[0].columns) {
        return new PrimaryKeyBuilder(config[0].columns, config[0].name);
    }
    return new PrimaryKeyBuilder(config);
}
class PrimaryKeyBuilder {
    static [alias$1.entityKind] = 'SQLitePrimaryKeyBuilder';
    /** @internal */
    columns;
    /** @internal */
    name;
    constructor(columns, name) {
        this.columns = columns;
        this.name = name;
    }
    /** @internal */
    build(table) {
        return new PrimaryKey(table, this.columns, this.name);
    }
}
class PrimaryKey {
    table;
    static [alias$1.entityKind] = 'SQLitePrimaryKey';
    columns;
    name;
    constructor(table, columns, name) {
        this.table = table;
        this.columns = columns;
        this.name = name;
    }
    getName() {
        return this.name ?? `${this.table[session.SQLiteTable.Symbol.Name]}_${this.columns.map((column) => column.name).join('_')}_pk`;
    }
}

function getTableConfig(table) {
    const columns = Object.values(table[session.SQLiteTable.Symbol.Columns]);
    const indexes = [];
    const checks = [];
    const primaryKeys = [];
    const uniqueConstraints = [];
    const foreignKeys = Object.values(table[session.SQLiteTable.Symbol.InlineForeignKeys]);
    const name = table[alias$1.Table.Symbol.Name];
    const extraConfigBuilder = table[session.SQLiteTable.Symbol.ExtraConfigBuilder];
    if (extraConfigBuilder !== undefined) {
        const extraConfig = extraConfigBuilder(table[session.SQLiteTable.Symbol.Columns]);
        for (const builder of Object.values(extraConfig)) {
            if (alias$1.is(builder, IndexBuilder)) {
                indexes.push(builder.build(table));
            }
            else if (alias$1.is(builder, CheckBuilder)) {
                checks.push(builder.build(table));
            }
            else if (alias$1.is(builder, session.UniqueConstraintBuilder)) {
                uniqueConstraints.push(builder.build(table));
            }
            else if (alias$1.is(builder, PrimaryKeyBuilder)) {
                primaryKeys.push(builder.build(table));
            }
            else if (alias$1.is(builder, session.ForeignKeyBuilder)) {
                foreignKeys.push(builder.build(table));
            }
        }
    }
    return {
        columns,
        indexes,
        foreignKeys,
        checks,
        primaryKeys,
        uniqueConstraints,
        name,
    };
}
function getViewConfig(view) {
    return {
        ...view[alias$1.ViewBaseConfig],
        ...view[session.SQLiteViewConfig],
    };
}

exports.BaseSQLiteDatabase = session.BaseSQLiteDatabase;
exports.ExecuteResultSync = session.ExecuteResultSync;
exports.ForeignKey = session.ForeignKey;
exports.ForeignKeyBuilder = session.ForeignKeyBuilder;
exports.InlineForeignKeys = session.InlineForeignKeys;
exports.ManualViewBuilder = session.ManualViewBuilder;
exports.QueryBuilder = session.QueryBuilder;
exports.SQLiteAsyncDialect = session.SQLiteAsyncDialect;
exports.SQLiteColumn = session.SQLiteColumn;
exports.SQLiteColumnBuilder = session.SQLiteColumnBuilder;
exports.SQLiteDeleteBase = session.SQLiteDeleteBase;
exports.SQLiteDialect = session.SQLiteDialect;
exports.SQLiteInsertBase = session.SQLiteInsertBase;
exports.SQLiteInsertBuilder = session.SQLiteInsertBuilder;
exports.SQLitePreparedQuery = session.SQLitePreparedQuery;
exports.SQLiteSelectBase = session.SQLiteSelectBase;
exports.SQLiteSelectBuilder = session.SQLiteSelectBuilder;
exports.SQLiteSelectQueryBuilderBase = session.SQLiteSelectQueryBuilderBase;
exports.SQLiteSession = session.SQLiteSession;
exports.SQLiteSyncDialect = session.SQLiteSyncDialect;
exports.SQLiteTable = session.SQLiteTable;
exports.SQLiteTransaction = session.SQLiteTransaction;
exports.SQLiteUpdateBase = session.SQLiteUpdateBase;
exports.SQLiteUpdateBuilder = session.SQLiteUpdateBuilder;
exports.SQLiteView = session.SQLiteView;
exports.SQLiteViewBase = session.SQLiteViewBase;
exports.UniqueConstraint = session.UniqueConstraint;
exports.UniqueConstraintBuilder = session.UniqueConstraintBuilder;
exports.UniqueOnConstraintBuilder = session.UniqueOnConstraintBuilder;
exports.ViewBuilder = session.ViewBuilder;
exports.ViewBuilderCore = session.ViewBuilderCore;
exports.foreignKey = session.foreignKey;
exports.sqliteTable = session.sqliteTable;
exports.sqliteTableCreator = session.sqliteTableCreator;
exports.sqliteView = session.sqliteView;
exports.unique = session.unique;
exports.uniqueKeyName = session.uniqueKeyName;
exports.view = session.view;
exports.Check = Check;
exports.CheckBuilder = CheckBuilder;
exports.Index = Index;
exports.IndexBuilder = IndexBuilder;
exports.IndexBuilderOn = IndexBuilderOn;
exports.PrimaryKey = PrimaryKey;
exports.PrimaryKeyBuilder = PrimaryKeyBuilder;
exports.SQLiteBaseInteger = SQLiteBaseInteger;
exports.SQLiteBaseIntegerBuilder = SQLiteBaseIntegerBuilder;
exports.SQLiteBigInt = SQLiteBigInt;
exports.SQLiteBigIntBuilder = SQLiteBigIntBuilder;
exports.SQLiteBlobBuffer = SQLiteBlobBuffer;
exports.SQLiteBlobBufferBuilder = SQLiteBlobBufferBuilder;
exports.SQLiteBlobJson = SQLiteBlobJson;
exports.SQLiteBlobJsonBuilder = SQLiteBlobJsonBuilder;
exports.SQLiteBoolean = SQLiteBoolean;
exports.SQLiteBooleanBuilder = SQLiteBooleanBuilder;
exports.SQLiteCustomColumn = SQLiteCustomColumn;
exports.SQLiteCustomColumnBuilder = SQLiteCustomColumnBuilder;
exports.SQLiteInteger = SQLiteInteger;
exports.SQLiteIntegerBuilder = SQLiteIntegerBuilder;
exports.SQLiteNumeric = SQLiteNumeric;
exports.SQLiteNumericBuilder = SQLiteNumericBuilder;
exports.SQLiteReal = SQLiteReal;
exports.SQLiteRealBuilder = SQLiteRealBuilder;
exports.SQLiteText = SQLiteText;
exports.SQLiteTextBuilder = SQLiteTextBuilder;
exports.SQLiteTextJson = SQLiteTextJson;
exports.SQLiteTextJsonBuilder = SQLiteTextJsonBuilder;
exports.SQLiteTimestamp = SQLiteTimestamp;
exports.SQLiteTimestampBuilder = SQLiteTimestampBuilder;
exports.alias = alias;
exports.blob = blob;
exports.check = check;
exports.customType = customType;
exports.getTableConfig = getTableConfig;
exports.getViewConfig = getViewConfig;
exports.index = index;
exports.int = int;
exports.integer = integer;
exports.numeric = numeric;
exports.primaryKey = primaryKey;
exports.real = real;
exports.text = text;
exports.uniqueIndex = uniqueIndex;
//# sourceMappingURL=index.cjs.map
