'use strict';

var index = require('../index.cjs');
var alias = require('../alias-aa343af5.cjs');
var session = require('../session-87c6f58b.cjs');

class SQLiteRemoteSession extends session.SQLiteSession {
    client;
    schema;
    static [alias.entityKind] = 'SQLiteRemoteSession';
    logger;
    constructor(client, dialect, schema, options = {}) {
        super(dialect);
        this.client = client;
        this.schema = schema;
        this.logger = options.logger ?? new index.NoopLogger();
    }
    prepareQuery(query, fields, executeMethod) {
        return new PreparedQuery(this.client, query, this.logger, fields, executeMethod);
    }
    async transaction(transaction, config) {
        const tx = new SQLiteProxyTransaction('async', this.dialect, this, this.schema);
        await this.run(alias.sql.raw(`begin${config?.behavior ? ' ' + config.behavior : ''}`));
        try {
            const result = await transaction(tx);
            await this.run(alias.sql `commit`);
            return result;
        }
        catch (err) {
            await this.run(alias.sql `rollback`);
            throw err;
        }
    }
}
class SQLiteProxyTransaction extends session.SQLiteTransaction {
    static [alias.entityKind] = 'SQLiteProxyTransaction';
    async transaction(transaction) {
        const savepointName = `sp${this.nestedIndex}`;
        const tx = new SQLiteProxyTransaction('async', this.dialect, this.session, this.schema, this.nestedIndex + 1);
        await this.session.run(alias.sql.raw(`savepoint ${savepointName}`));
        try {
            const result = await transaction(tx);
            await this.session.run(alias.sql.raw(`release savepoint ${savepointName}`));
            return result;
        }
        catch (err) {
            await this.session.run(alias.sql.raw(`rollback to savepoint ${savepointName}`));
            throw err;
        }
    }
}
class PreparedQuery extends session.SQLitePreparedQuery {
    client;
    logger;
    fields;
    static [alias.entityKind] = 'SQLiteProxyPreparedQuery';
    constructor(client, query, logger, fields, executeMethod) {
        super('async', executeMethod, query);
        this.client = client;
        this.logger = logger;
        this.fields = fields;
    }
    run(placeholderValues) {
        const params = alias.fillPlaceholders(this.query.params, placeholderValues ?? {});
        this.logger.logQuery(this.query.sql, params);
        return this.client(this.query.sql, params, 'run');
    }
    async all(placeholderValues) {
        const { fields, query, logger, joinsNotNullableMap } = this;
        const params = alias.fillPlaceholders(query.params, placeholderValues ?? {});
        logger.logQuery(query.sql, params);
        const { rows } = await this.client(query.sql, params, 'all');
        if (fields) {
            return rows.map((row) => alias.mapResultRow(fields, row, joinsNotNullableMap));
        }
        return rows;
    }
    async get(placeholderValues) {
        const { fields, query, logger, joinsNotNullableMap } = this;
        const params = alias.fillPlaceholders(query.params, placeholderValues ?? {});
        logger.logQuery(query.sql, params);
        const clientResult = await this.client(query.sql, params, 'get');
        if (fields) {
            if (clientResult.rows === undefined) {
                return undefined;
            }
            return alias.mapResultRow(fields, clientResult.rows, joinsNotNullableMap);
        }
        return clientResult.rows;
    }
    async values(placeholderValues) {
        const params = alias.fillPlaceholders(this.query.params, placeholderValues ?? {});
        this.logger.logQuery(this.query.sql, params);
        const clientResult = await this.client(this.query.sql, params, 'values');
        return clientResult.rows;
    }
}

function drizzle(callback, config = {}) {
    const dialect = new session.SQLiteAsyncDialect();
    let logger;
    if (config.logger === true) {
        logger = new index.DefaultLogger();
    }
    else if (config.logger !== false) {
        logger = config.logger;
    }
    let schema;
    if (config.schema) {
        const tablesConfig = alias.extractTablesRelationalConfig(config.schema, alias.createTableRelationsHelpers);
        schema = {
            fullSchema: config.schema,
            schema: tablesConfig.tables,
            tableNamesMap: tablesConfig.tableNamesMap,
        };
    }
    const session$1 = new SQLiteRemoteSession(callback, dialect, schema, { logger });
    return new session.BaseSQLiteDatabase('async', dialect, session$1, schema);
}

exports.PreparedQuery = PreparedQuery;
exports.SQLiteProxyTransaction = SQLiteProxyTransaction;
exports.SQLiteRemoteSession = SQLiteRemoteSession;
exports.drizzle = drizzle;
//# sourceMappingURL=index.cjs.map
